﻿using NUnit.Framework;
using Rhino.Mocks;
using System.IO;

namespace TDD.Mocking {
    [TestFixture]
    class OrderReaderRhinoTest {
        MockRepository repository;
        IFileReader mockFileReader;
        OrderReader orderReader;

        [SetUp]
        public void SetUp() {
            repository = new MockRepository();
            // Obsolete method: 
            // mockFileReader = repository.CreateMock<IFileReader>();
            mockFileReader = repository.StrictMock<IFileReader>();
            // mockFileReader = repository.DynamicMock<IFileReader>();
            orderReader = new OrderReader(mockFileReader);
        }

        // 4 styles - "classic"
        [Test]
        public void TestReadOrder0() {
            Expect.Call(mockFileReader.FileName = "record.csv");
            Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21");
            mockFileReader.Close();
            LastCall.Repeat.Once();
            repository.ReplayAll();

            Order order = orderReader.ReadOrder();
            repository.VerifyAll();

            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("FUBAR"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }

        [Test]
        public void TestReadOrder1() {
            using (repository.Record())
            //using (repository.Ordered())
            {
                // method which returns a value: 
                Expect.Call(mockFileReader.ReadLine()).Return("27412,FUBAR,876543.21");
                // property: 
                Expect.Call(mockFileReader.FileName = "record.csv");
                // void method: 
                Expect.Call(delegate { mockFileReader.Close(); });
            }

            //Order order;
            //using (repository.Playback())
            //{
            //    order = orderReader.ReadOrder();
            //}
            Order order = orderReader.ReadOrder();
            repository.VerifyAll();

            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("FUBAR"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }

        [Test]
        public void TestReadOrder2() {
            Order order = null;
            With.Mocks(repository).Expecting(delegate {
                Expect.Call(mockFileReader.FileName = "record.csv");
                Expect.Call(mockFileReader.ReadLine()).Return("27412,ABC,876543.21");
                // Lambda syntax: 
                mockFileReader.Expect(x => x.Close());
            })
            .Verify(delegate {
                order = orderReader.ReadOrder();
            });
            Assert.That(order.OrderId, Is.EqualTo(27412));
            Assert.That(order.Customer, Is.EqualTo("ABC"));
            Assert.That(order.Price, Is.EqualTo(876543.21));
        }

        //// Arrange, Act, Assert
        //[Test]
        //public void TestReadOrder3()
        //{
        //    // var mockFR = MockRepository.GenerateMock<IFileReader>();
        //    // If we want Ordered in AAA syntax: 
        //    // mockFR.GetMockRepository().Ordered();
        //    // For a stub: 
        //    var mockFR = MockRepository.GenerateStub<IFileReader>();

        //    // for a stub, property getter like this: 
        //    mockFR.FileName = "record.csv";
        //    // mockFR.Expect(x => x.FileName).Return("record.csv");

        //    string rv = "27412,FUBAR,876543.21";
        //    mockFR.Expect(x => x.ReadLine()).Return(rv);
        //    mockFR.Stub(x => x.Close());

        //    // mockFR.BackToRecord();
        //    // mockFR.Replay();

        //    Order order = orderReader.ReadOrder();

        //    Assert.That(order.OrderId, Is.EqualTo(27412));
        //    Assert.That(order.Customer, Is.EqualTo("FUBAR"));
        //    Assert.That(order.Price, Is.EqualTo(876543.21));

        //    // mockFR.VerifyAllExpectations();
        //}

        [Test, ExpectedException(typeof(IOException))]
        public void TestFileReaderThrowsUp() {
            using (repository.Record()) {
                Expect.Call(mockFileReader.FileName = "record.csv");
                // mock the file reader implementation throwing an IOEx:
                Expect.Call(delegate { mockFileReader.ReadLine(); }).Throw(new IOException("Bad file"));
                // we don't expect Close() to be called: 
                mockFileReader.Close();
                LastCall.Repeat.Never();
            }

            Order order;
            using (repository.Playback()) {
                order = orderReader.ReadOrder();
            }
        }
    }
}
